/***************************************************************************//**
* \file Console.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Console_PVT.h"

#if (Console_SCB_MODE_I2C_INC)
    #include "Console_I2C_PVT.h"
#endif /* (Console_SCB_MODE_I2C_INC) */

#if (Console_SCB_MODE_EZI2C_INC)
    #include "Console_EZI2C_PVT.h"
#endif /* (Console_SCB_MODE_EZI2C_INC) */

#if (Console_SCB_MODE_SPI_INC || Console_SCB_MODE_UART_INC)
    #include "Console_SPI_UART_PVT.h"
#endif /* (Console_SCB_MODE_SPI_INC || Console_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 Console_scbMode = Console_SCB_MODE_UNCONFIG;
    uint8 Console_scbEnableWake;
    uint8 Console_scbEnableIntr;

    /* I2C configuration variables */
    uint8 Console_mode;
    uint8 Console_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * Console_rxBuffer;
    uint8  Console_rxDataBits;
    uint32 Console_rxBufferSize;

    volatile uint8 * Console_txBuffer;
    uint8  Console_txDataBits;
    uint32 Console_txBufferSize;

    /* EZI2C configuration variables */
    uint8 Console_numberOfAddr;
    uint8 Console_subAddrSize;
#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/
/**
* \addtogroup group_general
* \{
*/

/** Console_initVar indicates whether the Console 
*  component has been initialized. The variable is initialized to 0 
*  and set to 1 the first time SCB_Start() is called. This allows 
*  the component to restart without reinitialization after the first 
*  call to the Console_Start() routine.
*
*  If re-initialization of the component is required, then the 
*  Console_Init() function can be called before the 
*  Console_Start() or Console_Enable() function.
*/
uint8 Console_initVar = 0u;


#if (! (Console_SCB_MODE_I2C_CONST_CFG || \
        Console_SCB_MODE_EZI2C_CONST_CFG))
    /** This global variable stores TX interrupt sources after 
    * Console_Stop() is called. Only these TX interrupt sources 
    * will be restored on a subsequent Console_Enable() call.
    */
    uint16 Console_IntrTxMask = 0u;
#endif /* (! (Console_SCB_MODE_I2C_CONST_CFG || \
              Console_SCB_MODE_EZI2C_CONST_CFG)) */
/** \} globals */

#if (Console_SCB_IRQ_INTERNAL)
#if !defined (CY_REMOVE_Console_CUSTOM_INTR_HANDLER)
    void (*Console_customIntrHandler)(void) = NULL;
#endif /* !defined (CY_REMOVE_Console_CUSTOM_INTR_HANDLER) */
#endif /* (Console_SCB_IRQ_INTERNAL) */


/***************************************
*    Private Function Prototypes
***************************************/

static void Console_ScbEnableIntr(void);
static void Console_ScbModeStop(void);
static void Console_ScbModePostEnable(void);


/*******************************************************************************
* Function Name: Console_Init
****************************************************************************//**
*
*  Initializes the Console component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  Console_I2CInit, Console_SpiInit, 
*  Console_UartInit or Console_EzI2CInit.
*
*******************************************************************************/
void Console_Init(void)
{
#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
    if (Console_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        Console_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif (Console_SCB_MODE_I2C_CONST_CFG)
    Console_I2CInit();

#elif (Console_SCB_MODE_SPI_CONST_CFG)
    Console_SpiInit();

#elif (Console_SCB_MODE_UART_CONST_CFG)
    Console_UartInit();

#elif (Console_SCB_MODE_EZI2C_CONST_CFG)
    Console_EzI2CInit();

#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: Console_Enable
****************************************************************************//**
*
*  Enables Console component operation: activates the hardware and 
*  internal interrupt. It also restores TX interrupt sources disabled after the 
*  Console_Stop() function was called (note that level-triggered TX 
*  interrupt sources remain disabled to not cause code lock-up).
*  For I2C and EZI2C modes the interrupt is internal and mandatory for 
*  operation. For SPI and UART modes the interrupt can be configured as none, 
*  internal or external.
*  The Console configuration should be not changed when the component
*  is enabled. Any configuration changes should be made after disabling the 
*  component.
*  When configuration is set to Unconfigured Console, the component 
*  must first be initialized to operate in one of the following configurations: 
*  I2C, SPI, UART or EZ I2C, using the mode-specific initialization API. 
*  Otherwise this function does not enable the component.
*
*******************************************************************************/
void Console_Enable(void)
{
#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if (!Console_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        Console_CTRL_REG |= Console_CTRL_ENABLED;

        Console_ScbEnableIntr();

        /* Call PostEnable function specific to current operation mode */
        Console_ScbModePostEnable();
    }
#else
    Console_CTRL_REG |= Console_CTRL_ENABLED;

    Console_ScbEnableIntr();

    /* Call PostEnable function specific to current operation mode */
    Console_ScbModePostEnable();
#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: Console_Start
****************************************************************************//**
*
*  Invokes Console_Init() and Console_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZI2C. Otherwise this function does not enable the component.
*
* \globalvars
*  Console_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void Console_Start(void)
{
    if (0u == Console_initVar)
    {
        Console_Init();
        Console_initVar = 1u; /* Component was initialized */
    }

    Console_Enable();
}


/*******************************************************************************
* Function Name: Console_Stop
****************************************************************************//**
*
*  Disables the Console component: disable the hardware and internal 
*  interrupt. It also disables all TX interrupt sources so as not to cause an 
*  unexpected interrupt trigger because after the component is enabled, the 
*  TX FIFO is empty.
*  Refer to the function Console_Enable() for the interrupt 
*  configuration details.
*  This function disables the SCB component without checking to see if 
*  communication is in progress. Before calling this function it may be 
*  necessary to check the status of communication to make sure communication 
*  is complete. If this is not done then communication could be stopped mid 
*  byte and corrupted data could result.
*
*******************************************************************************/
void Console_Stop(void)
{
#if (Console_SCB_IRQ_INTERNAL)
    Console_DisableInt();
#endif /* (Console_SCB_IRQ_INTERNAL) */

    /* Call Stop function specific to current operation mode */
    Console_ScbModeStop();

    /* Disable SCB IP */
    Console_CTRL_REG &= (uint32) ~Console_CTRL_ENABLED;

    /* Disable all TX interrupt sources so as not to cause an unexpected
    * interrupt trigger after the component will be enabled because the 
    * TX FIFO is empty.
    * For SCB IP v0, it is critical as it does not mask-out interrupt
    * sources when it is disabled. This can cause a code lock-up in the
    * interrupt handler because TX FIFO cannot be loaded after the block
    * is disabled.
    */
    Console_SetTxInterruptMode(Console_NO_INTR_SOURCES);

#if (Console_SCB_IRQ_INTERNAL)
    Console_ClearPendingInt();
#endif /* (Console_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: Console_SetRxFifoLevel
****************************************************************************//**
*
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
*  \param level: Level in the RX FIFO to generate RX level interrupt.
*   The range of valid level values is between 0 and RX FIFO depth - 1.
*
*******************************************************************************/
void Console_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = Console_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~Console_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (Console_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    Console_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: Console_SetTxFifoLevel
****************************************************************************//**
*
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has less entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
*  \param level: Level in the TX FIFO to generate TX level interrupt.
*   The range of valid level values is between 0 and TX FIFO depth - 1.
*
*******************************************************************************/
void Console_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = Console_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~Console_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (Console_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    Console_TX_FIFO_CTRL_REG = txFifoCtrl;
}


#if (Console_SCB_IRQ_INTERNAL)
    /*******************************************************************************
    * Function Name: Console_SetCustomInterruptHandler
    ****************************************************************************//**
    *
    *  Registers a function to be called by the internal interrupt handler.
    *  First the function that is registered is called, then the internal interrupt
    *  handler performs any operation such as software buffer management functions
    *  before the interrupt returns.  It is the user's responsibility not to break
    *  the software buffer operations. Only one custom handler is supported, which
    *  is the function provided by the most recent call.
    *  At the initialization time no custom handler is registered.
    *
    *  \param func: Pointer to the function to register.
    *        The value NULL indicates to remove the current custom interrupt
    *        handler.
    *
    *******************************************************************************/
    void Console_SetCustomInterruptHandler(void (*func)(void))
    {
    #if !defined (CY_REMOVE_Console_CUSTOM_INTR_HANDLER)
        Console_customIntrHandler = func; /* Register interrupt handler */
    #else
        if (NULL != func)
        {
            /* Suppress compiler warning */
        }
    #endif /* !defined (CY_REMOVE_Console_CUSTOM_INTR_HANDLER) */
    }
#endif /* (Console_SCB_IRQ_INTERNAL) */


/*******************************************************************************
* Function Name: Console_ScbModeEnableIntr
****************************************************************************//**
*
*  Enables an interrupt for a specific mode.
*
*******************************************************************************/
static void Console_ScbEnableIntr(void)
{
#if (Console_SCB_IRQ_INTERNAL)
    /* Enable interrupt in NVIC */
    #if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
        if (0u != Console_scbEnableIntr)
        {
            Console_EnableInt();
        }

    #else
        Console_EnableInt();

    #endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (Console_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: Console_ScbModePostEnable
****************************************************************************//**
*
*  Calls the PostEnable function for a specific operation mode.
*
*******************************************************************************/
static void Console_ScbModePostEnable(void)
{
#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
#if (!Console_CY_SCBIP_V1)
    if (Console_SCB_MODE_SPI_RUNTM_CFG)
    {
        Console_SpiPostEnable();
    }
    else if (Console_SCB_MODE_UART_RUNTM_CFG)
    {
        Console_UartPostEnable();
    }
    else
    {
        /* Unknown mode: do nothing */
    }
#endif /* (!Console_CY_SCBIP_V1) */

#elif (Console_SCB_MODE_SPI_CONST_CFG)
    Console_SpiPostEnable();

#elif (Console_SCB_MODE_UART_CONST_CFG)
    Console_UartPostEnable();

#else
    /* Unknown mode: do nothing */
#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: Console_ScbModeStop
****************************************************************************//**
*
*  Calls the Stop function for a specific operation mode.
*
*******************************************************************************/
static void Console_ScbModeStop(void)
{
#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
    if (Console_SCB_MODE_I2C_RUNTM_CFG)
    {
        Console_I2CStop();
    }
    else if (Console_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        Console_EzI2CStop();
    }
#if (!Console_CY_SCBIP_V1)
    else if (Console_SCB_MODE_SPI_RUNTM_CFG)
    {
        Console_SpiStop();
    }
    else if (Console_SCB_MODE_UART_RUNTM_CFG)
    {
        Console_UartStop();
    }
#endif /* (!Console_CY_SCBIP_V1) */
    else
    {
        /* Unknown mode: do nothing */
    }
#elif (Console_SCB_MODE_I2C_CONST_CFG)
    Console_I2CStop();

#elif (Console_SCB_MODE_EZI2C_CONST_CFG)
    Console_EzI2CStop();

#elif (Console_SCB_MODE_SPI_CONST_CFG)
    Console_SpiStop();

#elif (Console_SCB_MODE_UART_CONST_CFG)
    Console_UartStop();

#else
    /* Unknown mode: do nothing */
#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: Console_SetPins
    ****************************************************************************//**
    *
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    *  \param mode:      Mode of SCB operation.
    *  \param subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  \param uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    *******************************************************************************/
    void Console_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 pinsDm[Console_SCB_PINS_NUMBER];
        uint32 i;
        
    #if (!Console_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!Console_CY_SCBIP_V1) */
        
        uint32 hsiomSel[Console_SCB_PINS_NUMBER] = 
        {
            Console_RX_SDA_MOSI_HSIOM_SEL_GPIO,
            Console_TX_SCL_MISO_HSIOM_SEL_GPIO,
            0u,
            0u,
            0u,
            0u,
            0u,
        };

    #if (Console_CY_SCBIP_V1)
        /* Supress compiler warning. */
        if ((0u == subMode) || (0u == uartEnableMask))
        {
        }
    #endif /* (Console_CY_SCBIP_V1) */

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for (i = 0u; i < Console_SCB_PINS_NUMBER; i++)
        {
            pinsDm[i] = Console_PIN_DM_ALG_HIZ;
        }

        if ((Console_SCB_MODE_I2C   == mode) ||
            (Console_SCB_MODE_EZI2C == mode))
        {
        #if (Console_RX_SDA_MOSI_PIN)
            hsiomSel[Console_RX_SDA_MOSI_PIN_INDEX] = Console_RX_SDA_MOSI_HSIOM_SEL_I2C;
            pinsDm  [Console_RX_SDA_MOSI_PIN_INDEX] = Console_PIN_DM_OD_LO;
        #elif (Console_RX_WAKE_SDA_MOSI_PIN)
            hsiomSel[Console_RX_WAKE_SDA_MOSI_PIN_INDEX] = Console_RX_WAKE_SDA_MOSI_HSIOM_SEL_I2C;
            pinsDm  [Console_RX_WAKE_SDA_MOSI_PIN_INDEX] = Console_PIN_DM_OD_LO;
        #else
        #endif /* (Console_RX_SDA_MOSI_PIN) */
        
        #if (Console_TX_SCL_MISO_PIN)
            hsiomSel[Console_TX_SCL_MISO_PIN_INDEX] = Console_TX_SCL_MISO_HSIOM_SEL_I2C;
            pinsDm  [Console_TX_SCL_MISO_PIN_INDEX] = Console_PIN_DM_OD_LO;
        #endif /* (Console_TX_SCL_MISO_PIN) */
        }
    #if (!Console_CY_SCBIP_V1)
        else if (Console_SCB_MODE_SPI == mode)
        {
        #if (Console_RX_SDA_MOSI_PIN)
            hsiomSel[Console_RX_SDA_MOSI_PIN_INDEX] = Console_RX_SDA_MOSI_HSIOM_SEL_SPI;
        #elif (Console_RX_WAKE_SDA_MOSI_PIN)
            hsiomSel[Console_RX_WAKE_SDA_MOSI_PIN_INDEX] = Console_RX_WAKE_SDA_MOSI_HSIOM_SEL_SPI;
        #else
        #endif /* (Console_RX_SDA_MOSI_PIN) */
        
        #if (Console_TX_SCL_MISO_PIN)
            hsiomSel[Console_TX_SCL_MISO_PIN_INDEX] = Console_TX_SCL_MISO_HSIOM_SEL_SPI;
        #endif /* (Console_TX_SCL_MISO_PIN) */
        
        #if (Console_CTS_SCLK_PIN)
            hsiomSel[Console_CTS_SCLK_PIN_INDEX] = Console_CTS_SCLK_HSIOM_SEL_SPI;
        #endif /* (Console_CTS_SCLK_PIN) */

            if (Console_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[Console_RX_SDA_MOSI_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;
                pinsDm[Console_TX_SCL_MISO_PIN_INDEX] = Console_PIN_DM_STRONG;
                pinsDm[Console_CTS_SCLK_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;

            #if (Console_RTS_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[Console_RTS_SS0_PIN_INDEX] = Console_RTS_SS0_HSIOM_SEL_SPI;
                pinsDm  [Console_RTS_SS0_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;
            #endif /* (Console_RTS_SS0_PIN) */

            #if (Console_TX_SCL_MISO_PIN)
                /* Disable input buffer */
                 pinsInBuf |= Console_TX_SCL_MISO_PIN_MASK;
            #endif /* (Console_TX_SCL_MISO_PIN) */
            }
            else 
            {
                /* (Master) */
                pinsDm[Console_RX_SDA_MOSI_PIN_INDEX] = Console_PIN_DM_STRONG;
                pinsDm[Console_TX_SCL_MISO_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;
                pinsDm[Console_CTS_SCLK_PIN_INDEX] = Console_PIN_DM_STRONG;

            #if (Console_RTS_SS0_PIN)
                hsiomSel [Console_RTS_SS0_PIN_INDEX] = Console_RTS_SS0_HSIOM_SEL_SPI;
                pinsDm   [Console_RTS_SS0_PIN_INDEX] = Console_PIN_DM_STRONG;
                pinsInBuf |= Console_RTS_SS0_PIN_MASK;
            #endif /* (Console_RTS_SS0_PIN) */

            #if (Console_SS1_PIN)
                hsiomSel [Console_SS1_PIN_INDEX] = Console_SS1_HSIOM_SEL_SPI;
                pinsDm   [Console_SS1_PIN_INDEX] = Console_PIN_DM_STRONG;
                pinsInBuf |= Console_SS1_PIN_MASK;
            #endif /* (Console_SS1_PIN) */

            #if (Console_SS2_PIN)
                hsiomSel [Console_SS2_PIN_INDEX] = Console_SS2_HSIOM_SEL_SPI;
                pinsDm   [Console_SS2_PIN_INDEX] = Console_PIN_DM_STRONG;
                pinsInBuf |= Console_SS2_PIN_MASK;
            #endif /* (Console_SS2_PIN) */

            #if (Console_SS3_PIN)
                hsiomSel [Console_SS3_PIN_INDEX] = Console_SS3_HSIOM_SEL_SPI;
                pinsDm   [Console_SS3_PIN_INDEX] = Console_PIN_DM_STRONG;
                pinsInBuf |= Console_SS3_PIN_MASK;
            #endif /* (Console_SS3_PIN) */

                /* Disable input buffers */
            #if (Console_RX_SDA_MOSI_PIN)
                pinsInBuf |= Console_RX_SDA_MOSI_PIN_MASK;
            #elif (Console_RX_WAKE_SDA_MOSI_PIN)
                pinsInBuf |= Console_RX_WAKE_SDA_MOSI_PIN_MASK;
            #else
            #endif /* (Console_RX_SDA_MOSI_PIN) */

            #if (Console_CTS_SCLK_PIN)
                pinsInBuf |= Console_CTS_SCLK_PIN_MASK;
            #endif /* (Console_CTS_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if (Console_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
            #if (Console_TX_SCL_MISO_PIN)
                hsiomSel[Console_TX_SCL_MISO_PIN_INDEX] = Console_TX_SCL_MISO_HSIOM_SEL_UART;
                pinsDm  [Console_TX_SCL_MISO_PIN_INDEX] = Console_PIN_DM_OD_LO;
            #endif /* (Console_TX_SCL_MISO_PIN) */
            }
            else /* Standard or IrDA */
            {
                if (0u != (Console_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                #if (Console_RX_SDA_MOSI_PIN)
                    hsiomSel[Console_RX_SDA_MOSI_PIN_INDEX] = Console_RX_SDA_MOSI_HSIOM_SEL_UART;
                    pinsDm  [Console_RX_SDA_MOSI_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;
                #elif (Console_RX_WAKE_SDA_MOSI_PIN)
                    hsiomSel[Console_RX_WAKE_SDA_MOSI_PIN_INDEX] = Console_RX_WAKE_SDA_MOSI_HSIOM_SEL_UART;
                    pinsDm  [Console_RX_WAKE_SDA_MOSI_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;
                #else
                #endif /* (Console_RX_SDA_MOSI_PIN) */
                }

                if (0u != (Console_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                #if (Console_TX_SCL_MISO_PIN)
                    hsiomSel[Console_TX_SCL_MISO_PIN_INDEX] = Console_TX_SCL_MISO_HSIOM_SEL_UART;
                    pinsDm  [Console_TX_SCL_MISO_PIN_INDEX] = Console_PIN_DM_STRONG;
                    
                    /* Disable input buffer */
                    pinsInBuf |= Console_TX_SCL_MISO_PIN_MASK;
                #endif /* (Console_TX_SCL_MISO_PIN) */
                }

            #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
                if (Console_UART_MODE_STD == subMode)
                {
                    if (0u != (Console_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                    #if (Console_CTS_SCLK_PIN)
                        hsiomSel[Console_CTS_SCLK_PIN_INDEX] = Console_CTS_SCLK_HSIOM_SEL_UART;
                        pinsDm  [Console_CTS_SCLK_PIN_INDEX] = Console_PIN_DM_DIG_HIZ;
                    #endif /* (Console_CTS_SCLK_PIN) */
                    }

                    if (0u != (Console_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                    #if (Console_RTS_SS0_PIN)
                        hsiomSel[Console_RTS_SS0_PIN_INDEX] = Console_RTS_SS0_HSIOM_SEL_UART;
                        pinsDm  [Console_RTS_SS0_PIN_INDEX] = Console_PIN_DM_STRONG;
                        
                        /* Disable input buffer */
                        pinsInBuf |= Console_RTS_SS0_PIN_MASK;
                    #endif /* (Console_RTS_SS0_PIN) */
                    }
                }
            #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */
            }
        }
    #endif /* (!Console_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if (Console_RX_SDA_MOSI_PIN)
        Console_SET_HSIOM_SEL(Console_RX_SDA_MOSI_HSIOM_REG,
                                       Console_RX_SDA_MOSI_HSIOM_MASK,
                                       Console_RX_SDA_MOSI_HSIOM_POS,
                                        hsiomSel[Console_RX_SDA_MOSI_PIN_INDEX]);

        Console_uart_rx_i2c_sda_spi_mosi_SetDriveMode((uint8) pinsDm[Console_RX_SDA_MOSI_PIN_INDEX]);

        #if (!Console_CY_SCBIP_V1)
            Console_SET_INP_DIS(Console_uart_rx_i2c_sda_spi_mosi_INP_DIS,
                                         Console_uart_rx_i2c_sda_spi_mosi_MASK,
                                         (0u != (pinsInBuf & Console_RX_SDA_MOSI_PIN_MASK)));
        #endif /* (!Console_CY_SCBIP_V1) */
    
    #elif (Console_RX_WAKE_SDA_MOSI_PIN)
        Console_SET_HSIOM_SEL(Console_RX_WAKE_SDA_MOSI_HSIOM_REG,
                                       Console_RX_WAKE_SDA_MOSI_HSIOM_MASK,
                                       Console_RX_WAKE_SDA_MOSI_HSIOM_POS,
                                       hsiomSel[Console_RX_WAKE_SDA_MOSI_PIN_INDEX]);

        Console_uart_rx_wake_i2c_sda_spi_mosi_SetDriveMode((uint8)
                                                               pinsDm[Console_RX_WAKE_SDA_MOSI_PIN_INDEX]);

        Console_SET_INP_DIS(Console_uart_rx_wake_i2c_sda_spi_mosi_INP_DIS,
                                     Console_uart_rx_wake_i2c_sda_spi_mosi_MASK,
                                     (0u != (pinsInBuf & Console_RX_WAKE_SDA_MOSI_PIN_MASK)));

         /* Set interrupt on falling edge */
        Console_SET_INCFG_TYPE(Console_RX_WAKE_SDA_MOSI_INTCFG_REG,
                                        Console_RX_WAKE_SDA_MOSI_INTCFG_TYPE_MASK,
                                        Console_RX_WAKE_SDA_MOSI_INTCFG_TYPE_POS,
                                        Console_INTCFG_TYPE_FALLING_EDGE);
    #else
    #endif /* (Console_RX_WAKE_SDA_MOSI_PIN) */

    #if (Console_TX_SCL_MISO_PIN)
        Console_SET_HSIOM_SEL(Console_TX_SCL_MISO_HSIOM_REG,
                                       Console_TX_SCL_MISO_HSIOM_MASK,
                                       Console_TX_SCL_MISO_HSIOM_POS,
                                        hsiomSel[Console_TX_SCL_MISO_PIN_INDEX]);

        Console_uart_tx_i2c_scl_spi_miso_SetDriveMode((uint8) pinsDm[Console_TX_SCL_MISO_PIN_INDEX]);

    #if (!Console_CY_SCBIP_V1)
        Console_SET_INP_DIS(Console_uart_tx_i2c_scl_spi_miso_INP_DIS,
                                     Console_uart_tx_i2c_scl_spi_miso_MASK,
                                    (0u != (pinsInBuf & Console_TX_SCL_MISO_PIN_MASK)));
    #endif /* (!Console_CY_SCBIP_V1) */
    #endif /* (Console_RX_SDA_MOSI_PIN) */

    #if (Console_CTS_SCLK_PIN)
        Console_SET_HSIOM_SEL(Console_CTS_SCLK_HSIOM_REG,
                                       Console_CTS_SCLK_HSIOM_MASK,
                                       Console_CTS_SCLK_HSIOM_POS,
                                       hsiomSel[Console_CTS_SCLK_PIN_INDEX]);

        Console_uart_cts_spi_sclk_SetDriveMode((uint8) pinsDm[Console_CTS_SCLK_PIN_INDEX]);

        Console_SET_INP_DIS(Console_uart_cts_spi_sclk_INP_DIS,
                                     Console_uart_cts_spi_sclk_MASK,
                                     (0u != (pinsInBuf & Console_CTS_SCLK_PIN_MASK)));
    #endif /* (Console_CTS_SCLK_PIN) */

    #if (Console_RTS_SS0_PIN)
        Console_SET_HSIOM_SEL(Console_RTS_SS0_HSIOM_REG,
                                       Console_RTS_SS0_HSIOM_MASK,
                                       Console_RTS_SS0_HSIOM_POS,
                                       hsiomSel[Console_RTS_SS0_PIN_INDEX]);

        Console_uart_rts_spi_ss0_SetDriveMode((uint8) pinsDm[Console_RTS_SS0_PIN_INDEX]);

        Console_SET_INP_DIS(Console_uart_rts_spi_ss0_INP_DIS,
                                     Console_uart_rts_spi_ss0_MASK,
                                     (0u != (pinsInBuf & Console_RTS_SS0_PIN_MASK)));
    #endif /* (Console_RTS_SS0_PIN) */

    #if (Console_SS1_PIN)
        Console_SET_HSIOM_SEL(Console_SS1_HSIOM_REG,
                                       Console_SS1_HSIOM_MASK,
                                       Console_SS1_HSIOM_POS,
                                       hsiomSel[Console_SS1_PIN_INDEX]);

        Console_spi_ss1_SetDriveMode((uint8) pinsDm[Console_SS1_PIN_INDEX]);

        Console_SET_INP_DIS(Console_spi_ss1_INP_DIS,
                                     Console_spi_ss1_MASK,
                                     (0u != (pinsInBuf & Console_SS1_PIN_MASK)));
    #endif /* (Console_SS1_PIN) */

    #if (Console_SS2_PIN)
        Console_SET_HSIOM_SEL(Console_SS2_HSIOM_REG,
                                       Console_SS2_HSIOM_MASK,
                                       Console_SS2_HSIOM_POS,
                                       hsiomSel[Console_SS2_PIN_INDEX]);

        Console_spi_ss2_SetDriveMode((uint8) pinsDm[Console_SS2_PIN_INDEX]);

        Console_SET_INP_DIS(Console_spi_ss2_INP_DIS,
                                     Console_spi_ss2_MASK,
                                     (0u != (pinsInBuf & Console_SS2_PIN_MASK)));
    #endif /* (Console_SS2_PIN) */

    #if (Console_SS3_PIN)
        Console_SET_HSIOM_SEL(Console_SS3_HSIOM_REG,
                                       Console_SS3_HSIOM_MASK,
                                       Console_SS3_HSIOM_POS,
                                       hsiomSel[Console_SS3_PIN_INDEX]);

        Console_spi_ss3_SetDriveMode((uint8) pinsDm[Console_SS3_PIN_INDEX]);

        Console_SET_INP_DIS(Console_spi_ss3_INP_DIS,
                                     Console_spi_ss3_MASK,
                                     (0u != (pinsInBuf & Console_SS3_PIN_MASK)));
    #endif /* (Console_SS3_PIN) */
    }

#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: Console_I2CSlaveNackGeneration
    ****************************************************************************//**
    *
    *  Sets command to generate NACK to the address or data.
    *
    *******************************************************************************/
    void Console_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (Console_CTRL_REG & Console_CTRL_EC_AM_MODE)) &&
            (0u == (Console_I2C_CTRL_REG & Console_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            Console_CTRL_REG &= ~Console_CTRL_EC_AM_MODE;
            Console_CTRL_REG |=  Console_CTRL_EC_AM_MODE;
        }

        Console_I2C_SLAVE_CMD_REG = Console_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */


/* [] END OF FILE */
