/***************************************************************************//**
* \file Console_PM.c
* \version 3.20
*
* \brief
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Console.h"
#include "Console_PVT.h"

#if(Console_SCB_MODE_I2C_INC)
    #include "Console_I2C_PVT.h"
#endif /* (Console_SCB_MODE_I2C_INC) */

#if(Console_SCB_MODE_EZI2C_INC)
    #include "Console_EZI2C_PVT.h"
#endif /* (Console_SCB_MODE_EZI2C_INC) */

#if(Console_SCB_MODE_SPI_INC || Console_SCB_MODE_UART_INC)
    #include "Console_SPI_UART_PVT.h"
#endif /* (Console_SCB_MODE_SPI_INC || Console_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(Console_SCB_MODE_UNCONFIG_CONST_CFG || \
   (Console_SCB_MODE_I2C_CONST_CFG   && (!Console_I2C_WAKE_ENABLE_CONST))   || \
   (Console_SCB_MODE_EZI2C_CONST_CFG && (!Console_EZI2C_WAKE_ENABLE_CONST)) || \
   (Console_SCB_MODE_SPI_CONST_CFG   && (!Console_SPI_WAKE_ENABLE_CONST))   || \
   (Console_SCB_MODE_UART_CONST_CFG  && (!Console_UART_WAKE_ENABLE_CONST)))

    Console_BACKUP_STRUCT Console_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: Console_Sleep
****************************************************************************//**
*
*  Prepares the Console component to enter Deep Sleep.
*  The Enable wakeup from Deep Sleep Mode selection has an influence on this 
*  function implementation:
*  - Checked: configures the component to be wakeup source from Deep Sleep.
*  - Unchecked: stores the current component state (enabled or disabled) and 
*    disables the component. See SCB_Stop() function for details about component 
*    disabling.
*
*  Call the Console_Sleep() function before calling the 
*  CyPmSysDeepSleep() function. 
*  Refer to the PSoC Creator System Reference Guide for more information about 
*  power management functions and Low power section of this document for the 
*  selected mode.
*
*  This function should not be called before entering Sleep.
*
*******************************************************************************/
void Console_Sleep(void)
{
#if(Console_SCB_MODE_UNCONFIG_CONST_CFG)

    if(Console_SCB_WAKE_ENABLE_CHECK)
    {
        if(Console_SCB_MODE_I2C_RUNTM_CFG)
        {
            Console_I2CSaveConfig();
        }
        else if(Console_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            Console_EzI2CSaveConfig();
        }
    #if(!Console_CY_SCBIP_V1)
        else if(Console_SCB_MODE_SPI_RUNTM_CFG)
        {
            Console_SpiSaveConfig();
        }
        else if(Console_SCB_MODE_UART_RUNTM_CFG)
        {
            Console_UartSaveConfig();
        }
    #endif /* (!Console_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        Console_backup.enableState = (uint8) Console_GET_CTRL_ENABLED;

        if(0u != Console_backup.enableState)
        {
            Console_Stop();
        }
    }

#else

    #if (Console_SCB_MODE_I2C_CONST_CFG && Console_I2C_WAKE_ENABLE_CONST)
        Console_I2CSaveConfig();

    #elif (Console_SCB_MODE_EZI2C_CONST_CFG && Console_EZI2C_WAKE_ENABLE_CONST)
        Console_EzI2CSaveConfig();

    #elif (Console_SCB_MODE_SPI_CONST_CFG && Console_SPI_WAKE_ENABLE_CONST)
        Console_SpiSaveConfig();

    #elif (Console_SCB_MODE_UART_CONST_CFG && Console_UART_WAKE_ENABLE_CONST)
        Console_UartSaveConfig();

    #else

        Console_backup.enableState = (uint8) Console_GET_CTRL_ENABLED;

        if(0u != Console_backup.enableState)
        {
            Console_Stop();
        }

    #endif /* defined (Console_SCB_MODE_I2C_CONST_CFG) && (Console_I2C_WAKE_ENABLE_CONST) */

#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: Console_Wakeup
****************************************************************************//**
*
*  Prepares the Console component for Active mode operation after 
*  Deep Sleep.
*  The Enable wakeup from Deep Sleep Mode selection has influence on this 
*  function implementation:
*  - Checked: restores the component Active mode configuration.
*  - Unchecked: enables the component if it was enabled before enter Deep Sleep.
*
*  This function should not be called after exiting Sleep.
*
*  \sideeffect
*   Calling the Console_Wakeup() function without first calling the 
*   Console_Sleep() function may produce unexpected behavior.
*
*******************************************************************************/
void Console_Wakeup(void)
{
#if(Console_SCB_MODE_UNCONFIG_CONST_CFG)

    if(Console_SCB_WAKE_ENABLE_CHECK)
    {
        if(Console_SCB_MODE_I2C_RUNTM_CFG)
        {
            Console_I2CRestoreConfig();
        }
        else if(Console_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            Console_EzI2CRestoreConfig();
        }
    #if(!Console_CY_SCBIP_V1)
        else if(Console_SCB_MODE_SPI_RUNTM_CFG)
        {
            Console_SpiRestoreConfig();
        }
        else if(Console_SCB_MODE_UART_RUNTM_CFG)
        {
            Console_UartRestoreConfig();
        }
    #endif /* (!Console_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != Console_backup.enableState)
        {
            Console_Enable();
        }
    }

#else

    #if (Console_SCB_MODE_I2C_CONST_CFG  && Console_I2C_WAKE_ENABLE_CONST)
        Console_I2CRestoreConfig();

    #elif (Console_SCB_MODE_EZI2C_CONST_CFG && Console_EZI2C_WAKE_ENABLE_CONST)
        Console_EzI2CRestoreConfig();

    #elif (Console_SCB_MODE_SPI_CONST_CFG && Console_SPI_WAKE_ENABLE_CONST)
        Console_SpiRestoreConfig();

    #elif (Console_SCB_MODE_UART_CONST_CFG && Console_UART_WAKE_ENABLE_CONST)
        Console_UartRestoreConfig();

    #else

        if(0u != Console_backup.enableState)
        {
            Console_Enable();
        }

    #endif /* (Console_I2C_WAKE_ENABLE_CONST) */

#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
