/***************************************************************************//**
* \file Console_SPI_UART.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component in
*  SPI and UART modes.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Console_PVT.h"
#include "Console_SPI_UART_PVT.h"

/***************************************
*        SPI/UART Private Vars
***************************************/

#if(Console_INTERNAL_RX_SW_BUFFER_CONST)
    /* Start index to put data into the software receive buffer.*/
    volatile uint32 Console_rxBufferHead;
    /* Start index to get data from the software receive buffer.*/
    volatile uint32 Console_rxBufferTail;
    /**
    * \addtogroup group_globals
    * \{
    */
    /** Sets when internal software receive buffer overflow
    *  was occurred.
    */
    volatile uint8  Console_rxBufferOverflow;
    /** \} globals */
#endif /* (Console_INTERNAL_RX_SW_BUFFER_CONST) */

#if(Console_INTERNAL_TX_SW_BUFFER_CONST)
    /* Start index to put data into the software transmit buffer.*/
    volatile uint32 Console_txBufferHead;
    /* Start index to get data from the software transmit buffer.*/
    volatile uint32 Console_txBufferTail;
#endif /* (Console_INTERNAL_TX_SW_BUFFER_CONST) */

#if(Console_INTERNAL_RX_SW_BUFFER)
    /* Add one element to the buffer to receive full packet. One byte in receive buffer is always empty */
    volatile uint8 Console_rxBufferInternal[Console_INTERNAL_RX_BUFFER_SIZE];
#endif /* (Console_INTERNAL_RX_SW_BUFFER) */

#if(Console_INTERNAL_TX_SW_BUFFER)
    volatile uint8 Console_txBufferInternal[Console_TX_BUFFER_SIZE];
#endif /* (Console_INTERNAL_TX_SW_BUFFER) */


#if(Console_RX_DIRECTION)
    /*******************************************************************************
    * Function Name: Console_SpiUartReadRxData
    ****************************************************************************//**
    *
    *  Retrieves the next data element from the receive buffer.
    *   - RX software buffer is disabled: Returns data element retrieved from
    *     RX FIFO. Undefined data will be returned if the RX FIFO is empty.
    *   - RX software buffer is enabled: Returns data element from the software
    *     receive buffer. Zero value is returned if the software receive buffer
    *     is empty.
    *
    * \return
    *  Next data element from the receive buffer. 
    *  The amount of data bits to be received depends on RX data bits selection 
    *  (the data bit counting starts from LSB of return value).
    *
    * \globalvars
    *  Console_rxBufferHead - the start index to put data into the 
    *  software receive buffer.
    *  Console_rxBufferTail - the start index to get data from the 
    *  software receive buffer.
    *
    *******************************************************************************/
    uint32 Console_SpiUartReadRxData(void)
    {
        uint32 rxData = 0u;

    #if (Console_INTERNAL_RX_SW_BUFFER_CONST)
        uint32 locTail;
    #endif /* (Console_INTERNAL_RX_SW_BUFFER_CONST) */

        #if (Console_CHECK_RX_SW_BUFFER)
        {
            if (Console_rxBufferHead != Console_rxBufferTail)
            {
                /* There is data in RX software buffer */

                /* Calculate index to read from */
                locTail = (Console_rxBufferTail + 1u);

                if (Console_INTERNAL_RX_BUFFER_SIZE == locTail)
                {
                    locTail = 0u;
                }

                /* Get data from RX software buffer */
                rxData = Console_GetWordFromRxBuffer(locTail);

                /* Change index in the buffer */
                Console_rxBufferTail = locTail;

                #if (Console_CHECK_UART_RTS_CONTROL_FLOW)
                {
                    /* Check if RX Not Empty is disabled in the interrupt */
                    if (0u == (Console_INTR_RX_MASK_REG & Console_INTR_RX_NOT_EMPTY))
                    {
                        /* Enable RX Not Empty interrupt source to continue
                        * receiving data into software buffer.
                        */
                        Console_INTR_RX_MASK_REG |= Console_INTR_RX_NOT_EMPTY;
                    }
                }
                #endif

            }
        }
        #else
        {
            /* Read data from RX FIFO */
            rxData = Console_RX_FIFO_RD_REG;
        }
        #endif

        return (rxData);
    }


    /*******************************************************************************
    * Function Name: Console_SpiUartGetRxBufferSize
    ****************************************************************************//**
    *
    *  Returns the number of received data elements in the receive buffer.
    *   - RX software buffer disabled: returns the number of used entries in
    *     RX FIFO.
    *   - RX software buffer enabled: returns the number of elements which were
    *     placed in the receive buffer. This does not include the hardware RX FIFO.
    *
    * \return
    *  Number of received data elements.
    *
    * \globalvars
    *  Console_rxBufferHead - the start index to put data into the 
    *  software receive buffer.
    *  Console_rxBufferTail - the start index to get data from the 
    *  software receive buffer.
    *
    *******************************************************************************/
    uint32 Console_SpiUartGetRxBufferSize(void)
    {
        uint32 size;
    #if (Console_INTERNAL_RX_SW_BUFFER_CONST)
        uint32 locHead;
    #endif /* (Console_INTERNAL_RX_SW_BUFFER_CONST) */

        #if (Console_CHECK_RX_SW_BUFFER)
        {
            locHead = Console_rxBufferHead;

            if(locHead >= Console_rxBufferTail)
            {
                size = (locHead - Console_rxBufferTail);
            }
            else
            {
                size = (locHead + (Console_INTERNAL_RX_BUFFER_SIZE - Console_rxBufferTail));
            }
        }
        #else
        {
            size = Console_GET_RX_FIFO_ENTRIES;
        }
        #endif

        return (size);
    }


    /*******************************************************************************
    * Function Name: Console_SpiUartClearRxBuffer
    ****************************************************************************//**
    *
    *  Clears the receive buffer and RX FIFO.
    *
    * \globalvars
    *  Console_rxBufferHead - the start index to put data into the 
    *  software receive buffer.
    *  Console_rxBufferTail - the start index to get data from the 
    *  software receive buffer.
    *
    *******************************************************************************/
    void Console_SpiUartClearRxBuffer(void)
    {
        #if (Console_CHECK_RX_SW_BUFFER)
        {
            /* Lock from component interruption */
            Console_DisableInt();

            /* Flush RX software buffer */
            Console_rxBufferHead = Console_rxBufferTail;
            Console_rxBufferOverflow = 0u;

            Console_CLEAR_RX_FIFO;
            Console_ClearRxInterruptSource(Console_INTR_RX_ALL);

            #if (Console_CHECK_UART_RTS_CONTROL_FLOW)
            {
                /* Enable RX Not Empty interrupt source to continue receiving
                * data into software buffer.
                */
                Console_INTR_RX_MASK_REG |= Console_INTR_RX_NOT_EMPTY;
            }
            #endif
            
            /* Release lock */
            Console_EnableInt();
        }
        #else
        {
            Console_CLEAR_RX_FIFO;
        }
        #endif
    }

#endif /* (Console_RX_DIRECTION) */


#if(Console_TX_DIRECTION)
    /*******************************************************************************
    * Function Name: Console_SpiUartWriteTxData
    ****************************************************************************//**
    *
    *  Places a data entry into the transmit buffer to be sent at the next available
    *  bus time.
    *  This function is blocking and waits until there is space available to put the
    *  requested data in the transmit buffer.
    *
    *  \param txDataByte: the data to be transmitted.
    *   The amount of data bits to be transmitted depends on TX data bits selection 
    *   (the data bit counting starts from LSB of txDataByte).
    *
    * \globalvars
    *  Console_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  Console_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    void Console_SpiUartWriteTxData(uint32 txData)
    {
    #if (Console_INTERNAL_TX_SW_BUFFER_CONST)
        uint32 locHead;
    #endif /* (Console_INTERNAL_TX_SW_BUFFER_CONST) */

        #if (Console_CHECK_TX_SW_BUFFER)
        {
            /* Put data directly into the TX FIFO */
            if ((Console_txBufferHead == Console_txBufferTail) &&
                (Console_SPI_UART_FIFO_SIZE != Console_GET_TX_FIFO_ENTRIES))
            {
                /* TX software buffer is empty: put data directly in TX FIFO */
                Console_TX_FIFO_WR_REG = txData;
            }
            /* Put data into TX software buffer */
            else
            {
                /* Head index to put data */
                locHead = (Console_txBufferHead + 1u);

                /* Adjust TX software buffer index */
                if (Console_TX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                /* Wait for space in TX software buffer */
                while (locHead == Console_txBufferTail)
                {
                }

                /* TX software buffer has at least one room */

                /* Clear old status of INTR_TX_NOT_FULL. It sets at the end of transfer when TX FIFO is empty. */
                Console_ClearTxInterruptSource(Console_INTR_TX_NOT_FULL);

                Console_PutWordInTxBuffer(locHead, txData);

                Console_txBufferHead = locHead;

                /* Check if TX Not Full is disabled in interrupt */
                if (0u == (Console_INTR_TX_MASK_REG & Console_INTR_TX_NOT_FULL))
                {
                    /* Enable TX Not Full interrupt source to transmit from software buffer */
                    Console_INTR_TX_MASK_REG |= (uint32) Console_INTR_TX_NOT_FULL;
                }
            }
        }
        #else
        {
            /* Wait until TX FIFO has space to put data element */
            while (Console_SPI_UART_FIFO_SIZE == Console_GET_TX_FIFO_ENTRIES)
            {
            }

            Console_TX_FIFO_WR_REG = txData;
        }
        #endif
    }


    /*******************************************************************************
    * Function Name: Console_SpiUartPutArray
    ****************************************************************************//**
    *
    *  Places an array of data into the transmit buffer to be sent.
    *  This function is blocking and waits until there is a space available to put
    *  all the requested data in the transmit buffer. The array size can be greater
    *  than transmit buffer size.
    *
    * \param wrBuf: pointer to an array of data to be placed in transmit buffer. 
    *  The width of the data to be transmitted depends on TX data width selection 
    *  (the data bit counting starts from LSB for each array element).
    * \param count: number of data elements to be placed in the transmit buffer.
    *
    * \globalvars
    *  Console_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  Console_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    void Console_SpiUartPutArray(const uint8 wrBuf[], uint32 count)
    {
        uint32 i;

        for (i=0u; i < count; i++)
        {
            Console_SpiUartWriteTxData((uint32) wrBuf[i]);
        }
    }


    /*******************************************************************************
    * Function Name: Console_SpiUartGetTxBufferSize
    ****************************************************************************//**
    *
    *  Returns the number of elements currently in the transmit buffer.
    *   - TX software buffer is disabled: returns the number of used entries in
    *     TX FIFO.
    *   - TX software buffer is enabled: returns the number of elements currently
    *     used in the transmit buffer. This number does not include used entries in
    *     the TX FIFO. The transmit buffer size is zero until the TX FIFO is
    *     not full.
    *
    * \return
    *  Number of data elements ready to transmit.
    *
    * \globalvars
    *  Console_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  Console_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    uint32 Console_SpiUartGetTxBufferSize(void)
    {
        uint32 size;
    #if (Console_INTERNAL_TX_SW_BUFFER_CONST)
        uint32 locTail;
    #endif /* (Console_INTERNAL_TX_SW_BUFFER_CONST) */

        #if (Console_CHECK_TX_SW_BUFFER)
        {
            /* Get current Tail index */
            locTail = Console_txBufferTail;

            if (Console_txBufferHead >= locTail)
            {
                size = (Console_txBufferHead - locTail);
            }
            else
            {
                size = (Console_txBufferHead + (Console_TX_BUFFER_SIZE - locTail));
            }
        }
        #else
        {
            size = Console_GET_TX_FIFO_ENTRIES;
        }
        #endif

        return (size);
    }


    /*******************************************************************************
    * Function Name: Console_SpiUartClearTxBuffer
    ****************************************************************************//**
    *
    *  Clears the transmit buffer and TX FIFO.
    *
    * \globalvars
    *  Console_txBufferHead - the start index to put data into the 
    *  software transmit buffer.
    *  Console_txBufferTail - start index to get data from the software
    *  transmit buffer.
    *
    *******************************************************************************/
    void Console_SpiUartClearTxBuffer(void)
    {
        #if (Console_CHECK_TX_SW_BUFFER)
        {
            /* Lock from component interruption */
            Console_DisableInt();

            /* Flush TX software buffer */
            Console_txBufferHead = Console_txBufferTail;

            Console_INTR_TX_MASK_REG &= (uint32) ~Console_INTR_TX_NOT_FULL;
            Console_CLEAR_TX_FIFO;
            Console_ClearTxInterruptSource(Console_INTR_TX_ALL);

            /* Release lock */
            Console_EnableInt();
        }
        #else
        {
            Console_CLEAR_TX_FIFO;
        }
        #endif
    }

#endif /* (Console_TX_DIRECTION) */


/*******************************************************************************
* Function Name: Console_SpiUartDisableIntRx
****************************************************************************//**
*
*  Disables the RX interrupt sources.
*
*  \return
*   Returns the RX interrupt sources enabled before the function call.
*
*******************************************************************************/
uint32 Console_SpiUartDisableIntRx(void)
{
    uint32 intSource;

    intSource = Console_GetRxInterruptMode();

    Console_SetRxInterruptMode(Console_NO_INTR_SOURCES);

    return (intSource);
}


/*******************************************************************************
* Function Name: Console_SpiUartDisableIntTx
****************************************************************************//**
*
*  Disables TX interrupt sources.
*
*  \return
*   Returns TX interrupt sources enabled before function call.
*
*******************************************************************************/
uint32 Console_SpiUartDisableIntTx(void)
{
    uint32 intSourceMask;

    intSourceMask = Console_GetTxInterruptMode();

    Console_SetTxInterruptMode(Console_NO_INTR_SOURCES);

    return (intSourceMask);
}


#if(Console_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: Console_PutWordInRxBuffer
    ****************************************************************************//**
    *
    *  Stores a byte/word into the RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \param index:      index to store data byte/word in the RX buffer.
    *  \param rxDataByte: byte/word to store.
    *
    *******************************************************************************/
    void Console_PutWordInRxBuffer(uint32 idx, uint32 rxDataByte)
    {
        /* Put data in buffer */
        if (Console_ONE_BYTE_WIDTH == Console_rxDataBits)
        {
            Console_rxBuffer[idx] = ((uint8) rxDataByte);
        }
        else
        {
            Console_rxBuffer[(uint32)(idx << 1u)]      = LO8(LO16(rxDataByte));
            Console_rxBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(rxDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: Console_GetWordFromRxBuffer
    ****************************************************************************//**
    *
    *  Reads byte/word from RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \return
    *   Returns byte/word read from RX buffer.
    *
    *******************************************************************************/
    uint32 Console_GetWordFromRxBuffer(uint32 idx)
    {
        uint32 value;

        if (Console_ONE_BYTE_WIDTH == Console_rxDataBits)
        {
            value = Console_rxBuffer[idx];
        }
        else
        {
            value  = (uint32) Console_rxBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32)Console_rxBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return (value);
    }


    /*******************************************************************************
    * Function Name: Console_PutWordInTxBuffer
    ****************************************************************************//**
    *
    *  Stores byte/word into the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \param idx:        index to store data byte/word in the TX buffer.
    *  \param txDataByte: byte/word to store.
    *
    *******************************************************************************/
    void Console_PutWordInTxBuffer(uint32 idx, uint32 txDataByte)
    {
        /* Put data in buffer */
        if (Console_ONE_BYTE_WIDTH == Console_txDataBits)
        {
            Console_txBuffer[idx] = ((uint8) txDataByte);
        }
        else
        {
            Console_txBuffer[(uint32)(idx << 1u)]      = LO8(LO16(txDataByte));
            Console_txBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(txDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: Console_GetWordFromTxBuffer
    ****************************************************************************//**
    *
    *  Reads byte/word from the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    *  \param idx: index to get data byte/word from the TX buffer.
    *
    *  \return
    *   Returns byte/word read from the TX buffer.
    *
    *******************************************************************************/
    uint32 Console_GetWordFromTxBuffer(uint32 idx)
    {
        uint32 value;

        if (Console_ONE_BYTE_WIDTH == Console_txDataBits)
        {
            value = (uint32) Console_txBuffer[idx];
        }
        else
        {
            value  = (uint32) Console_txBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32) Console_txBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return (value);
    }

#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
