/***************************************************************************//**
* \file Console_SPI_UART_INT.c
* \version 3.20
*
* \brief
*  This file provides the source code to the Interrupt Service Routine for
*  the SCB Component in SPI and UART modes.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Console_PVT.h"
#include "Console_SPI_UART_PVT.h"


#if (Console_SCB_IRQ_INTERNAL)
/*******************************************************************************
* Function Name: Console_SPI_UART_ISR
****************************************************************************//**
*
*  Handles the Interrupt Service Routine for the SCB SPI or UART modes.
*
*******************************************************************************/
CY_ISR(Console_SPI_UART_ISR)
{
#if (Console_INTERNAL_RX_SW_BUFFER_CONST)
    uint32 locHead;
#endif /* (Console_INTERNAL_RX_SW_BUFFER_CONST) */

#if (Console_INTERNAL_TX_SW_BUFFER_CONST)
    uint32 locTail;
#endif /* (Console_INTERNAL_TX_SW_BUFFER_CONST) */

#ifdef Console_SPI_UART_ISR_ENTRY_CALLBACK
    Console_SPI_UART_ISR_EntryCallback();
#endif /* Console_SPI_UART_ISR_ENTRY_CALLBACK */

    if (NULL != Console_customIntrHandler)
    {
        Console_customIntrHandler();
    }

    #if(Console_CHECK_SPI_WAKE_ENABLE)
    {
        /* Clear SPI wakeup source */
        Console_ClearSpiExtClkInterruptSource(Console_INTR_SPI_EC_WAKE_UP);
    }
    #endif

    #if (Console_CHECK_RX_SW_BUFFER)
    {
        if (Console_CHECK_INTR_RX_MASKED(Console_INTR_RX_NOT_EMPTY))
        {
            do
            {
                /* Move local head index */
                locHead = (Console_rxBufferHead + 1u);

                /* Adjust local head index */
                if (Console_INTERNAL_RX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                if (locHead == Console_rxBufferTail)
                {
                    #if (Console_CHECK_UART_RTS_CONTROL_FLOW)
                    {
                        /* There is no space in the software buffer - disable the
                        * RX Not Empty interrupt source. The data elements are
                        * still being received into the RX FIFO until the RTS signal
                        * stops the transmitter. After the data element is read from the
                        * buffer, the RX Not Empty interrupt source is enabled to
                        * move the next data element in the software buffer.
                        */
                        Console_INTR_RX_MASK_REG &= ~Console_INTR_RX_NOT_EMPTY;
                        break;
                    }
                    #else
                    {
                        /* Overflow: through away received data element */
                        (void) Console_RX_FIFO_RD_REG;
                        Console_rxBufferOverflow = (uint8) Console_INTR_RX_OVERFLOW;
                    }
                    #endif
                }
                else
                {
                    /* Store received data */
                    Console_PutWordInRxBuffer(locHead, Console_RX_FIFO_RD_REG);

                    /* Move head index */
                    Console_rxBufferHead = locHead;
                }
            }
            while(0u != Console_GET_RX_FIFO_ENTRIES);

            Console_ClearRxInterruptSource(Console_INTR_RX_NOT_EMPTY);
        }
    }
    #endif


    #if (Console_CHECK_TX_SW_BUFFER)
    {
        if (Console_CHECK_INTR_TX_MASKED(Console_INTR_TX_NOT_FULL))
        {
            do
            {
                /* Check for room in TX software buffer */
                if (Console_txBufferHead != Console_txBufferTail)
                {
                    /* Move local tail index */
                    locTail = (Console_txBufferTail + 1u);

                    /* Adjust local tail index */
                    if (Console_TX_BUFFER_SIZE == locTail)
                    {
                        locTail = 0u;
                    }

                    /* Put data into TX FIFO */
                    Console_TX_FIFO_WR_REG = Console_GetWordFromTxBuffer(locTail);

                    /* Move tail index */
                    Console_txBufferTail = locTail;
                }
                else
                {
                    /* TX software buffer is empty: complete transfer */
                    Console_DISABLE_INTR_TX(Console_INTR_TX_NOT_FULL);
                    break;
                }
            }
            while (Console_SPI_UART_FIFO_SIZE != Console_GET_TX_FIFO_ENTRIES);

            Console_ClearTxInterruptSource(Console_INTR_TX_NOT_FULL);
        }
    }
    #endif

#ifdef Console_SPI_UART_ISR_EXIT_CALLBACK
    Console_SPI_UART_ISR_ExitCallback();
#endif /* Console_SPI_UART_ISR_EXIT_CALLBACK */

}

#endif /* (Console_SCB_IRQ_INTERNAL) */


/* [] END OF FILE */
