/***************************************************************************//**
* \file Console_UART.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component in
*  UART mode.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Console_PVT.h"
#include "Console_SPI_UART_PVT.h"


#if (Console_UART_WAKE_ENABLE_CONST && Console_UART_RX_WAKEUP_IRQ)
    /**
    * \addtogroup group_globals
    * \{
    */
    /** This global variable determines whether to enable Skip Start
    * functionality when Console_Sleep() function is called:
    * 0  disable, other values  enable. Default value is 1.
    * It is only available when Enable wakeup from Deep Sleep Mode is enabled.
    */
    uint8 Console_skipStart = 1u;
    /** \} globals */
#endif /* (Console_UART_WAKE_ENABLE_CONST && Console_UART_RX_WAKEUP_IRQ) */

#if(Console_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Configuration Structure Initialization
    ***************************************/

    const Console_UART_INIT_STRUCT Console_configUart =
    {
        Console_UART_SUB_MODE,
        Console_UART_DIRECTION,
        Console_UART_DATA_BITS_NUM,
        Console_UART_PARITY_TYPE,
        Console_UART_STOP_BITS_NUM,
        Console_UART_OVS_FACTOR,
        Console_UART_IRDA_LOW_POWER,
        Console_UART_MEDIAN_FILTER_ENABLE,
        Console_UART_RETRY_ON_NACK,
        Console_UART_IRDA_POLARITY,
        Console_UART_DROP_ON_PARITY_ERR,
        Console_UART_DROP_ON_FRAME_ERR,
        Console_UART_WAKE_ENABLE,
        0u,
        NULL,
        0u,
        NULL,
        Console_UART_MP_MODE_ENABLE,
        Console_UART_MP_ACCEPT_ADDRESS,
        Console_UART_MP_RX_ADDRESS,
        Console_UART_MP_RX_ADDRESS_MASK,
        (uint32) Console_SCB_IRQ_INTERNAL,
        Console_UART_INTR_RX_MASK,
        Console_UART_RX_TRIGGER_LEVEL,
        Console_UART_INTR_TX_MASK,
        Console_UART_TX_TRIGGER_LEVEL,
        (uint8) Console_UART_BYTE_MODE_ENABLE,
        (uint8) Console_UART_CTS_ENABLE,
        (uint8) Console_UART_CTS_POLARITY,
        (uint8) Console_UART_RTS_POLARITY,
        (uint8) Console_UART_RTS_FIFO_LEVEL
    };


    /*******************************************************************************
    * Function Name: Console_UartInit
    ****************************************************************************//**
    *
    *  Configures the Console for UART operation.
    *
    *  This function is intended specifically to be used when the Console
    *  configuration is set to Unconfigured Console in the customizer.
    *  After initializing the Console in UART mode using this function,
    *  the component can be enabled using the Console_Start() or
    * Console_Enable() function.
    *  This function uses a pointer to a structure that provides the configuration
    *  settings. This structure contains the same information that would otherwise
    *  be provided by the customizer settings.
    *
    *  \param config: pointer to a structure that contains the following list of
    *   fields. These fields match the selections available in the customizer.
    *   Refer to the customizer for further description of the settings.
    *
    *******************************************************************************/
    void Console_UartInit(const Console_UART_INIT_STRUCT *config)
    {
        uint32 pinsConfig;

        if (NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due to bad function parameter */
        }
        else
        {
            /* Get direction to configure UART pins: TX, RX or TX+RX */
            pinsConfig  = config->direction;

        #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
            /* Add RTS and CTS pins to configure */
            pinsConfig |= (0u != config->rtsRxFifoLevel) ? (Console_UART_RTS_PIN_ENABLE) : (0u);
            pinsConfig |= (0u != config->enableCts)      ? (Console_UART_CTS_PIN_ENABLE) : (0u);
        #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

            /* Configure pins */
            Console_SetPins(Console_SCB_MODE_UART, config->mode, pinsConfig);

            /* Store internal configuration */
            Console_scbMode       = (uint8) Console_SCB_MODE_UART;
            Console_scbEnableWake = (uint8) config->enableWake;
            Console_scbEnableIntr = (uint8) config->enableInterrupt;

            /* Set RX direction internal variables */
            Console_rxBuffer      =         config->rxBuffer;
            Console_rxDataBits    = (uint8) config->dataBits;
            Console_rxBufferSize  = (uint8) config->rxBufferSize;

            /* Set TX direction internal variables */
            Console_txBuffer      =         config->txBuffer;
            Console_txDataBits    = (uint8) config->dataBits;
            Console_txBufferSize  = (uint8) config->txBufferSize;

            /* Configure UART interface */
            if(Console_UART_MODE_IRDA == config->mode)
            {
                /* OVS settings: IrDA */
                Console_CTRL_REG  = ((0u != config->enableIrdaLowPower) ?
                                                (Console_UART_GET_CTRL_OVS_IRDA_LP(config->oversample)) :
                                                (Console_CTRL_OVS_IRDA_OVS16));
            }
            else
            {
                /* OVS settings: UART and SmartCard */
                Console_CTRL_REG  = Console_GET_CTRL_OVS(config->oversample);
            }

            Console_CTRL_REG     |= Console_GET_CTRL_BYTE_MODE  (config->enableByteMode)      |
                                             Console_GET_CTRL_ADDR_ACCEPT(config->multiprocAcceptAddr) |
                                             Console_CTRL_UART;

            /* Configure sub-mode: UART, SmartCard or IrDA */
            Console_UART_CTRL_REG = Console_GET_UART_CTRL_MODE(config->mode);

            /* Configure RX direction */
            Console_UART_RX_CTRL_REG = Console_GET_UART_RX_CTRL_MODE(config->stopBits)              |
                                        Console_GET_UART_RX_CTRL_POLARITY(config->enableInvertedRx)          |
                                        Console_GET_UART_RX_CTRL_MP_MODE(config->enableMultiproc)            |
                                        Console_GET_UART_RX_CTRL_DROP_ON_PARITY_ERR(config->dropOnParityErr) |
                                        Console_GET_UART_RX_CTRL_DROP_ON_FRAME_ERR(config->dropOnFrameErr);

            if(Console_UART_PARITY_NONE != config->parity)
            {
               Console_UART_RX_CTRL_REG |= Console_GET_UART_RX_CTRL_PARITY(config->parity) |
                                                    Console_UART_RX_CTRL_PARITY_ENABLED;
            }

            Console_RX_CTRL_REG      = Console_GET_RX_CTRL_DATA_WIDTH(config->dataBits)       |
                                                Console_GET_RX_CTRL_MEDIAN(config->enableMedianFilter) |
                                                Console_GET_UART_RX_CTRL_ENABLED(config->direction);

            Console_RX_FIFO_CTRL_REG = Console_GET_RX_FIFO_CTRL_TRIGGER_LEVEL(config->rxTriggerLevel);

            /* Configure MP address */
            Console_RX_MATCH_REG     = Console_GET_RX_MATCH_ADDR(config->multiprocAddr) |
                                                Console_GET_RX_MATCH_MASK(config->multiprocAddrMask);

            /* Configure RX direction */
            Console_UART_TX_CTRL_REG = Console_GET_UART_TX_CTRL_MODE(config->stopBits) |
                                                Console_GET_UART_TX_CTRL_RETRY_NACK(config->enableRetryNack);

            if(Console_UART_PARITY_NONE != config->parity)
            {
               Console_UART_TX_CTRL_REG |= Console_GET_UART_TX_CTRL_PARITY(config->parity) |
                                                    Console_UART_TX_CTRL_PARITY_ENABLED;
            }

            Console_TX_CTRL_REG      = Console_GET_TX_CTRL_DATA_WIDTH(config->dataBits)    |
                                                Console_GET_UART_TX_CTRL_ENABLED(config->direction);

            Console_TX_FIFO_CTRL_REG = Console_GET_TX_FIFO_CTRL_TRIGGER_LEVEL(config->txTriggerLevel);

        #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
            Console_UART_FLOW_CTRL_REG = Console_GET_UART_FLOW_CTRL_CTS_ENABLE(config->enableCts) | \
                                            Console_GET_UART_FLOW_CTRL_CTS_POLARITY (config->ctsPolarity)  | \
                                            Console_GET_UART_FLOW_CTRL_RTS_POLARITY (config->rtsPolarity)  | \
                                            Console_GET_UART_FLOW_CTRL_TRIGGER_LEVEL(config->rtsRxFifoLevel);
        #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

            /* Configure interrupt with UART handler but do not enable it */
            CyIntDisable    (Console_ISR_NUMBER);
            CyIntSetPriority(Console_ISR_NUMBER, Console_ISR_PRIORITY);
            (void) CyIntSetVector(Console_ISR_NUMBER, &Console_SPI_UART_ISR);

            /* Configure WAKE interrupt */
        #if(Console_UART_RX_WAKEUP_IRQ)
            CyIntDisable    (Console_RX_WAKE_ISR_NUMBER);
            CyIntSetPriority(Console_RX_WAKE_ISR_NUMBER, Console_RX_WAKE_ISR_PRIORITY);
            (void) CyIntSetVector(Console_RX_WAKE_ISR_NUMBER, &Console_UART_WAKEUP_ISR);
        #endif /* (Console_UART_RX_WAKEUP_IRQ) */

            /* Configure interrupt sources */
            Console_INTR_I2C_EC_MASK_REG = Console_NO_INTR_SOURCES;
            Console_INTR_SPI_EC_MASK_REG = Console_NO_INTR_SOURCES;
            Console_INTR_SLAVE_MASK_REG  = Console_NO_INTR_SOURCES;
            Console_INTR_MASTER_MASK_REG = Console_NO_INTR_SOURCES;
            Console_INTR_RX_MASK_REG     = config->rxInterruptMask;
            Console_INTR_TX_MASK_REG     = config->txInterruptMask;
        
            /* Configure TX interrupt sources to restore. */
            Console_IntrTxMask = LO16(Console_INTR_TX_MASK_REG);

            /* Clear RX buffer indexes */
            Console_rxBufferHead     = 0u;
            Console_rxBufferTail     = 0u;
            Console_rxBufferOverflow = 0u;

            /* Clear TX buffer indexes */
            Console_txBufferHead = 0u;
            Console_txBufferTail = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: Console_UartInit
    ****************************************************************************//**
    *
    *  Configures the SCB for the UART operation.
    *
    *******************************************************************************/
    void Console_UartInit(void)
    {
        /* Configure UART interface */
        Console_CTRL_REG = Console_UART_DEFAULT_CTRL;

        /* Configure sub-mode: UART, SmartCard or IrDA */
        Console_UART_CTRL_REG = Console_UART_DEFAULT_UART_CTRL;

        /* Configure RX direction */
        Console_UART_RX_CTRL_REG = Console_UART_DEFAULT_UART_RX_CTRL;
        Console_RX_CTRL_REG      = Console_UART_DEFAULT_RX_CTRL;
        Console_RX_FIFO_CTRL_REG = Console_UART_DEFAULT_RX_FIFO_CTRL;
        Console_RX_MATCH_REG     = Console_UART_DEFAULT_RX_MATCH_REG;

        /* Configure TX direction */
        Console_UART_TX_CTRL_REG = Console_UART_DEFAULT_UART_TX_CTRL;
        Console_TX_CTRL_REG      = Console_UART_DEFAULT_TX_CTRL;
        Console_TX_FIFO_CTRL_REG = Console_UART_DEFAULT_TX_FIFO_CTRL;

    #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
        Console_UART_FLOW_CTRL_REG = Console_UART_DEFAULT_FLOW_CTRL;
    #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

        /* Configure interrupt with UART handler but do not enable it */
    #if(Console_SCB_IRQ_INTERNAL)
        CyIntDisable    (Console_ISR_NUMBER);
        CyIntSetPriority(Console_ISR_NUMBER, Console_ISR_PRIORITY);
        (void) CyIntSetVector(Console_ISR_NUMBER, &Console_SPI_UART_ISR);
    #endif /* (Console_SCB_IRQ_INTERNAL) */

        /* Configure WAKE interrupt */
    #if(Console_UART_RX_WAKEUP_IRQ)
        CyIntDisable    (Console_RX_WAKE_ISR_NUMBER);
        CyIntSetPriority(Console_RX_WAKE_ISR_NUMBER, Console_RX_WAKE_ISR_PRIORITY);
        (void) CyIntSetVector(Console_RX_WAKE_ISR_NUMBER, &Console_UART_WAKEUP_ISR);
    #endif /* (Console_UART_RX_WAKEUP_IRQ) */

        /* Configure interrupt sources */
        Console_INTR_I2C_EC_MASK_REG = Console_UART_DEFAULT_INTR_I2C_EC_MASK;
        Console_INTR_SPI_EC_MASK_REG = Console_UART_DEFAULT_INTR_SPI_EC_MASK;
        Console_INTR_SLAVE_MASK_REG  = Console_UART_DEFAULT_INTR_SLAVE_MASK;
        Console_INTR_MASTER_MASK_REG = Console_UART_DEFAULT_INTR_MASTER_MASK;
        Console_INTR_RX_MASK_REG     = Console_UART_DEFAULT_INTR_RX_MASK;
        Console_INTR_TX_MASK_REG     = Console_UART_DEFAULT_INTR_TX_MASK;
    
        /* Configure TX interrupt sources to restore. */
        Console_IntrTxMask = LO16(Console_INTR_TX_MASK_REG);

    #if(Console_INTERNAL_RX_SW_BUFFER_CONST)
        Console_rxBufferHead     = 0u;
        Console_rxBufferTail     = 0u;
        Console_rxBufferOverflow = 0u;
    #endif /* (Console_INTERNAL_RX_SW_BUFFER_CONST) */

    #if(Console_INTERNAL_TX_SW_BUFFER_CONST)
        Console_txBufferHead = 0u;
        Console_txBufferTail = 0u;
    #endif /* (Console_INTERNAL_TX_SW_BUFFER_CONST) */
    }
#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: Console_UartPostEnable
****************************************************************************//**
*
*  Restores HSIOM settings for the UART output pins (TX and/or RTS) to be
*  controlled by the SCB UART.
*
*******************************************************************************/
void Console_UartPostEnable(void)
{
#if (Console_SCB_MODE_UNCONFIG_CONST_CFG)
    #if (Console_TX_SCL_MISO_PIN)
        if (Console_CHECK_TX_SCL_MISO_PIN_USED)
        {
            /* Set SCB UART to drive the output pin */
            Console_SET_HSIOM_SEL(Console_TX_SCL_MISO_HSIOM_REG, Console_TX_SCL_MISO_HSIOM_MASK,
                                           Console_TX_SCL_MISO_HSIOM_POS, Console_TX_SCL_MISO_HSIOM_SEL_UART);
        }
    #endif /* (Console_TX_SCL_MISO_PIN_PIN) */

    #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
        #if (Console_RTS_SS0_PIN)
            if (Console_CHECK_RTS_SS0_PIN_USED)
            {
                /* Set SCB UART to drive the output pin */
                Console_SET_HSIOM_SEL(Console_RTS_SS0_HSIOM_REG, Console_RTS_SS0_HSIOM_MASK,
                                               Console_RTS_SS0_HSIOM_POS, Console_RTS_SS0_HSIOM_SEL_UART);
            }
        #endif /* (Console_RTS_SS0_PIN) */
    #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

#else
    #if (Console_UART_TX_PIN)
         /* Set SCB UART to drive the output pin */
        Console_SET_HSIOM_SEL(Console_TX_HSIOM_REG, Console_TX_HSIOM_MASK,
                                       Console_TX_HSIOM_POS, Console_TX_HSIOM_SEL_UART);
    #endif /* (Console_UART_TX_PIN) */

    #if (Console_UART_RTS_PIN)
        /* Set SCB UART to drive the output pin */
        Console_SET_HSIOM_SEL(Console_RTS_HSIOM_REG, Console_RTS_HSIOM_MASK,
                                       Console_RTS_HSIOM_POS, Console_RTS_HSIOM_SEL_UART);
    #endif /* (Console_UART_RTS_PIN) */
#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */

    /* Restore TX interrupt sources. */
    Console_SetTxInterruptMode(Console_IntrTxMask);
}


/*******************************************************************************
* Function Name: Console_UartStop
****************************************************************************//**
*
*  Changes the HSIOM settings for the UART output pins (TX and/or RTS) to keep
*  them inactive after the block is disabled. The output pins are controlled by
*  the GPIO data register. Also, the function disables the skip start feature
*  to not cause it to trigger after the component is enabled.
*
*******************************************************************************/
void Console_UartStop(void)
{
#if(Console_SCB_MODE_UNCONFIG_CONST_CFG)
    #if (Console_TX_SCL_MISO_PIN)
        if (Console_CHECK_TX_SCL_MISO_PIN_USED)
        {
            /* Set GPIO to drive output pin */
            Console_SET_HSIOM_SEL(Console_TX_SCL_MISO_HSIOM_REG, Console_TX_SCL_MISO_HSIOM_MASK,
                                           Console_TX_SCL_MISO_HSIOM_POS, Console_TX_SCL_MISO_HSIOM_SEL_GPIO);
        }
    #endif /* (Console_TX_SCL_MISO_PIN_PIN) */

    #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
        #if (Console_RTS_SS0_PIN)
            if (Console_CHECK_RTS_SS0_PIN_USED)
            {
                /* Set output pin state after block is disabled */
                Console_uart_rts_spi_ss0_Write(Console_GET_UART_RTS_INACTIVE);

                /* Set GPIO to drive output pin */
                Console_SET_HSIOM_SEL(Console_RTS_SS0_HSIOM_REG, Console_RTS_SS0_HSIOM_MASK,
                                               Console_RTS_SS0_HSIOM_POS, Console_RTS_SS0_HSIOM_SEL_GPIO);
            }
        #endif /* (Console_RTS_SS0_PIN) */
    #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

#else
    #if (Console_UART_TX_PIN)
        /* Set GPIO to drive output pin */
        Console_SET_HSIOM_SEL(Console_TX_HSIOM_REG, Console_TX_HSIOM_MASK,
                                       Console_TX_HSIOM_POS, Console_TX_HSIOM_SEL_GPIO);
    #endif /* (Console_UART_TX_PIN) */

    #if (Console_UART_RTS_PIN)
        /* Set output pin state after block is disabled */
        Console_rts_Write(Console_GET_UART_RTS_INACTIVE);

        /* Set GPIO to drive output pin */
        Console_SET_HSIOM_SEL(Console_RTS_HSIOM_REG, Console_RTS_HSIOM_MASK,
                                       Console_RTS_HSIOM_POS, Console_RTS_HSIOM_SEL_GPIO);
    #endif /* (Console_UART_RTS_PIN) */

#endif /* (Console_SCB_MODE_UNCONFIG_CONST_CFG) */

#if (Console_UART_WAKE_ENABLE_CONST)
    /* Disable skip start feature used for wakeup */
    Console_UART_RX_CTRL_REG &= (uint32) ~Console_UART_RX_CTRL_SKIP_START;
#endif /* (Console_UART_WAKE_ENABLE_CONST) */

    /* Store TX interrupt sources (exclude level triggered). */
    Console_IntrTxMask = LO16(Console_GetTxInterruptMode() & Console_INTR_UART_TX_RESTORE);
}


/*******************************************************************************
* Function Name: Console_UartSetRxAddress
****************************************************************************//**
*
*  Sets the hardware detectable receiver address for the UART in the
*  Multiprocessor mode.
*
*  \param address: Address for hardware address detection.
*
*******************************************************************************/
void Console_UartSetRxAddress(uint32 address)
{
     uint32 matchReg;

    matchReg = Console_RX_MATCH_REG;

    matchReg &= ((uint32) ~Console_RX_MATCH_ADDR_MASK); /* Clear address bits */
    matchReg |= ((uint32)  (address & Console_RX_MATCH_ADDR_MASK)); /* Set address  */

    Console_RX_MATCH_REG = matchReg;
}


/*******************************************************************************
* Function Name: Console_UartSetRxAddressMask
****************************************************************************//**
*
*  Sets the hardware address mask for the UART in the Multiprocessor mode.
*
*  \param addressMask: Address mask.
*   - Bit value 0  excludes bit from address comparison.
*   - Bit value 1  the bit needs to match with the corresponding bit
*     of the address.
*
*******************************************************************************/
void Console_UartSetRxAddressMask(uint32 addressMask)
{
    uint32 matchReg;

    matchReg = Console_RX_MATCH_REG;

    matchReg &= ((uint32) ~Console_RX_MATCH_MASK_MASK); /* Clear address mask bits */
    matchReg |= ((uint32) (addressMask << Console_RX_MATCH_MASK_POS));

    Console_RX_MATCH_REG = matchReg;
}


#if(Console_UART_RX_DIRECTION)
    /*******************************************************************************
    * Function Name: Console_UartGetChar
    ****************************************************************************//**
    *
    *  Retrieves next data element from receive buffer.
    *  This function is designed for ASCII characters and returns a char where
    *  1 to 255 are valid characters and 0 indicates an error occurred or no data
    *  is present.
    *  - RX software buffer is disabled: Returns data element retrieved from RX
    *    FIFO.
    *  - RX software buffer is enabled: Returns data element from the software
    *    receive buffer.
    *
    *  \return
    *   Next data element from the receive buffer. ASCII character values from
    *   1 to 255 are valid. A returned zero signifies an error condition or no
    *   data available.
    *
    *  \sideeffect
    *   The errors bits may not correspond with reading characters due to
    *   RX FIFO and software buffer usage.
    *   RX software buffer is enabled: The internal software buffer overflow
    *   is not treated as an error condition.
    *   Check Console_rxBufferOverflow to capture that error condition.
    *
    *******************************************************************************/
    uint32 Console_UartGetChar(void)
    {
        uint32 rxData = 0u;

        /* Reads data only if there is data to read */
        if (0u != Console_SpiUartGetRxBufferSize())
        {
            rxData = Console_SpiUartReadRxData();
        }

        if (Console_CHECK_INTR_RX(Console_INTR_RX_ERR))
        {
            rxData = 0u; /* Error occurred: returns zero */
            Console_ClearRxInterruptSource(Console_INTR_RX_ERR);
        }

        return (rxData);
    }


    /*******************************************************************************
    * Function Name: Console_UartGetByte
    ****************************************************************************//**
    *
    *  Retrieves the next data element from the receive buffer, returns the
    *  received byte and error condition.
    *   - The RX software buffer is disabled: returns the data element retrieved
    *     from the RX FIFO. Undefined data will be returned if the RX FIFO is
    *     empty.
    *   - The RX software buffer is enabled: returns data element from the
    *     software receive buffer.
    *
    *  \return
    *   Bits 7-0 contain the next data element from the receive buffer and
    *   other bits contain the error condition.
    *   - Console_UART_RX_OVERFLOW - Attempt to write to a full
    *     receiver FIFO.
    *   - Console_UART_RX_UNDERFLOW	Attempt to read from an empty
    *     receiver FIFO.
    *   - Console_UART_RX_FRAME_ERROR - UART framing error detected.
    *   - Console_UART_RX_PARITY_ERROR - UART parity error detected.
    *
    *  \sideeffect
    *   The errors bits may not correspond with reading characters due to
    *   RX FIFO and software buffer usage.
    *   RX software buffer is enabled: The internal software buffer overflow
    *   is not treated as an error condition.
    *   Check Console_rxBufferOverflow to capture that error condition.
    *
    *******************************************************************************/
    uint32 Console_UartGetByte(void)
    {
        uint32 rxData;
        uint32 tmpStatus;

        #if (Console_CHECK_RX_SW_BUFFER)
        {
            Console_DisableInt();
        }
        #endif

        if (0u != Console_SpiUartGetRxBufferSize())
        {
            /* Enables interrupt to receive more bytes: at least one byte is in
            * buffer.
            */
            #if (Console_CHECK_RX_SW_BUFFER)
            {
                Console_EnableInt();
            }
            #endif

            /* Get received byte */
            rxData = Console_SpiUartReadRxData();
        }
        else
        {
            /* Reads a byte directly from RX FIFO: underflow is raised in the
            * case of empty. Otherwise the first received byte will be read.
            */
            rxData = Console_RX_FIFO_RD_REG;


            /* Enables interrupt to receive more bytes. */
            #if (Console_CHECK_RX_SW_BUFFER)
            {

                /* The byte has been read from RX FIFO. Clear RX interrupt to
                * not involve interrupt handler when RX FIFO is empty.
                */
                Console_ClearRxInterruptSource(Console_INTR_RX_NOT_EMPTY);

                Console_EnableInt();
            }
            #endif
        }

        /* Get and clear RX error mask */
        tmpStatus = (Console_GetRxInterruptSource() & Console_INTR_RX_ERR);
        Console_ClearRxInterruptSource(Console_INTR_RX_ERR);

        /* Puts together data and error status:
        * MP mode and accept address: 9th bit is set to notify mark.
        */
        rxData |= ((uint32) (tmpStatus << 8u));

        return (rxData);
    }


    #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
        /*******************************************************************************
        * Function Name: Console_UartSetRtsPolarity
        ****************************************************************************//**
        *
        *  Sets active polarity of RTS output signal.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *  \param polarity: Active polarity of RTS output signal.
        *   - Console_UART_RTS_ACTIVE_LOW  - RTS signal is active low.
        *   - Console_UART_RTS_ACTIVE_HIGH - RTS signal is active high.
        *
        *******************************************************************************/
        void Console_UartSetRtsPolarity(uint32 polarity)
        {
            if(0u != polarity)
            {
                Console_UART_FLOW_CTRL_REG |= (uint32)  Console_UART_FLOW_CTRL_RTS_POLARITY;
            }
            else
            {
                Console_UART_FLOW_CTRL_REG &= (uint32) ~Console_UART_FLOW_CTRL_RTS_POLARITY;
            }
        }


        /*******************************************************************************
        * Function Name: Console_UartSetRtsFifoLevel
        ****************************************************************************//**
        *
        *  Sets level in the RX FIFO for RTS signal activation.
        *  While the RX FIFO has fewer entries than the RX FIFO level the RTS signal
        *  remains active, otherwise the RTS signal becomes inactive.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *  \param level: Level in the RX FIFO for RTS signal activation.
        *   The range of valid level values is between 0 and RX FIFO depth - 1.
        *   Setting level value to 0 disables RTS signal activation.
        *
        *******************************************************************************/
        void Console_UartSetRtsFifoLevel(uint32 level)
        {
            uint32 uartFlowCtrl;

            uartFlowCtrl = Console_UART_FLOW_CTRL_REG;

            uartFlowCtrl &= ((uint32) ~Console_UART_FLOW_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
            uartFlowCtrl |= ((uint32) (Console_UART_FLOW_CTRL_TRIGGER_LEVEL_MASK & level));

            Console_UART_FLOW_CTRL_REG = uartFlowCtrl;
        }
    #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

#endif /* (Console_UART_RX_DIRECTION) */


#if(Console_UART_TX_DIRECTION)
    /*******************************************************************************
    * Function Name: Console_UartPutString
    ****************************************************************************//**
    *
    *  Places a NULL terminated string in the transmit buffer to be sent at the
    *  next available bus time.
    *  This function is blocking and waits until there is a space available to put
    *  requested data in transmit buffer.
    *
    *  \param string: pointer to the null terminated string array to be placed in the
    *   transmit buffer.
    *
    *******************************************************************************/
    void Console_UartPutString(const char8 string[])
    {
        uint32 bufIndex;

        bufIndex = 0u;

        /* Blocks the control flow until all data has been sent */
        while(string[bufIndex] != ((char8) 0))
        {
            Console_UartPutChar((uint32) string[bufIndex]);
            bufIndex++;
        }
    }


    /*******************************************************************************
    * Function Name: Console_UartPutCRLF
    ****************************************************************************//**
    *
    *  Places byte of data followed by a carriage return (0x0D) and line feed
    *  (0x0A) in the transmit buffer.
    *  This function is blocking and waits until there is a space available to put
    *  all requested data in transmit buffer.
    *
    *  \param txDataByte: the data to be transmitted.
    *
    *******************************************************************************/
    void Console_UartPutCRLF(uint32 txDataByte)
    {
        Console_UartPutChar(txDataByte);  /* Blocks control flow until all data has been sent */
        Console_UartPutChar(0x0Du);       /* Blocks control flow until all data has been sent */
        Console_UartPutChar(0x0Au);       /* Blocks control flow until all data has been sent */
    }


    #if !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1)
        /*******************************************************************************
        * Function Name: ConsoleSCB_UartEnableCts
        ****************************************************************************//**
        *
        *  Enables usage of CTS input signal by the UART transmitter.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *******************************************************************************/
        void Console_UartEnableCts(void)
        {
            Console_UART_FLOW_CTRL_REG |= (uint32)  Console_UART_FLOW_CTRL_CTS_ENABLE;
        }


        /*******************************************************************************
        * Function Name: Console_UartDisableCts
        ****************************************************************************//**
        *
        *  Disables usage of CTS input signal by the UART transmitter.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *******************************************************************************/
        void Console_UartDisableCts(void)
        {
            Console_UART_FLOW_CTRL_REG &= (uint32) ~Console_UART_FLOW_CTRL_CTS_ENABLE;
        }


        /*******************************************************************************
        * Function Name: Console_UartSetCtsPolarity
        ****************************************************************************//**
        *
        *  Sets active polarity of CTS input signal.
        *  Only available for PSoC 4100 BLE / PSoC 4200 BLE / PSoC 4100M / PSoC 4200M /
        *  PSoC 4200L / PSoC 4000S / PSoC 4100S / PSoC Analog Coprocessor devices.
        *
        *  \param polarity: Active polarity of CTS output signal.
        *   - Console_UART_CTS_ACTIVE_LOW  - CTS signal is active low.
        *   - Console_UART_CTS_ACTIVE_HIGH - CTS signal is active high.
        *
        *******************************************************************************/
        void Console_UartSetCtsPolarity(uint32 polarity)
        {
            if (0u != polarity)
            {
                Console_UART_FLOW_CTRL_REG |= (uint32)  Console_UART_FLOW_CTRL_CTS_POLARITY;
            }
            else
            {
                Console_UART_FLOW_CTRL_REG &= (uint32) ~Console_UART_FLOW_CTRL_CTS_POLARITY;
            }
        }
    #endif /* !(Console_CY_SCBIP_V0 || Console_CY_SCBIP_V1) */

#endif /* (Console_UART_TX_DIRECTION) */


#if (Console_UART_WAKE_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: Console_UartSaveConfig
    ****************************************************************************//**
    *
    *  Clears and enables an interrupt on a falling edge of the Rx input. The GPIO
    *  interrupt does not track in the active mode, therefore requires to be 
    *  cleared by this API.
    *
    *******************************************************************************/
    void Console_UartSaveConfig(void)
    {
    #if (Console_UART_RX_WAKEUP_IRQ)
        /* Set SKIP_START if requested (set by default). */
        if (0u != Console_skipStart)
        {
            Console_UART_RX_CTRL_REG |= (uint32)  Console_UART_RX_CTRL_SKIP_START;
        }
        else
        {
            Console_UART_RX_CTRL_REG &= (uint32) ~Console_UART_RX_CTRL_SKIP_START;
        }
        
        /* Clear RX GPIO interrupt status and pending interrupt in NVIC because
        * falling edge on RX line occurs while UART communication in active mode.
        * Enable interrupt: next interrupt trigger should wakeup device.
        */
        Console_CLEAR_UART_RX_WAKE_INTR;
        Console_RxWakeClearPendingInt();
        Console_RxWakeEnableInt();
    #endif /* (Console_UART_RX_WAKEUP_IRQ) */
    }


    /*******************************************************************************
    * Function Name: Console_UartRestoreConfig
    ****************************************************************************//**
    *
    *  Disables the RX GPIO interrupt. Until this function is called the interrupt
    *  remains active and triggers on every falling edge of the UART RX line.
    *
    *******************************************************************************/
    void Console_UartRestoreConfig(void)
    {
    #if (Console_UART_RX_WAKEUP_IRQ)
        /* Disable interrupt: no more triggers in active mode */
        Console_RxWakeDisableInt();
    #endif /* (Console_UART_RX_WAKEUP_IRQ) */
    }


    #if (Console_UART_RX_WAKEUP_IRQ)
        /*******************************************************************************
        * Function Name: Console_UART_WAKEUP_ISR
        ****************************************************************************//**
        *
        *  Handles the Interrupt Service Routine for the SCB UART mode GPIO wakeup
        *  event. This event is configured to trigger on a falling edge of the RX line.
        *
        *******************************************************************************/
        CY_ISR(Console_UART_WAKEUP_ISR)
        {
        #ifdef Console_UART_WAKEUP_ISR_ENTRY_CALLBACK
            Console_UART_WAKEUP_ISR_EntryCallback();
        #endif /* Console_UART_WAKEUP_ISR_ENTRY_CALLBACK */

            Console_CLEAR_UART_RX_WAKE_INTR;

        #ifdef Console_UART_WAKEUP_ISR_EXIT_CALLBACK
            Console_UART_WAKEUP_ISR_ExitCallback();
        #endif /* Console_UART_WAKEUP_ISR_EXIT_CALLBACK */
        }
    #endif /* (Console_UART_RX_WAKEUP_IRQ) */
#endif /* (Console_UART_RX_WAKEUP_IRQ) */


/* [] END OF FILE */
